Index: include/ao/ao.h
===================================================================
RCS file: /usr/local/cvsroot/ao/include/ao/ao.h,v
retrieving revision 1.8
diff -u -3 -p -r1.8 ao.h
--- include/ao/ao.h	2000/12/30 05:03:24	1.8
+++ include/ao/ao.h	2001/01/19 11:31:36
@@ -61,6 +61,7 @@ typedef struct ao_functions_s {
 	ao_internal_t *(*open)(uint_32 bits, uint_32 rate, uint_32 channels, ao_option_t *options);
 	void (*play)(ao_internal_t *state, void* output_samples, uint_32 num_bytes);
 	void (*close)(ao_internal_t *state);
+	int (*get_latency)(ao_internal_t *state);
 } ao_functions_t;
 
 typedef struct ao_device_s {
@@ -96,6 +97,9 @@ void ao_close(ao_device_t *device);
 
 /* misc functions */
 int ao_is_big_endian(void);
+
+/* returns the number of bytes buffered by the driver / output device */
+int ao_get_latency(ao_device_t *device);
 
 #ifdef __cplusplus
 }
Index: src/ao_null.c
===================================================================
RCS file: /usr/local/cvsroot/ao/src/ao_null.c,v
retrieving revision 1.2
diff -u -3 -p -r1.2 ao_null.c
--- src/ao_null.c	2000/10/30 00:46:41	1.2
+++ src/ao_null.c	2001/01/19 11:31:36
@@ -73,9 +73,15 @@ static ao_info_t *ao_null_get_driver_inf
 	return &ao_null_info;
 }
 
+static int ao_null_get_latency(void)
+{
+	return 0;
+}
+
 ao_functions_t ao_null = {
 	ao_null_get_driver_info,
 	ao_null_open,
 	ao_null_play,
-	ao_null_close
+	ao_null_close,
+	ao_null_get_latency
 };
Index: src/ao_wav.c
===================================================================
RCS file: /usr/local/cvsroot/ao/src/ao_wav.c,v
retrieving revision 1.5
diff -u -3 -p -r1.5 ao_wav.c
--- src/ao_wav.c	2000/12/24 06:21:51	1.5
+++ src/ao_wav.c	2001/01/19 11:31:38
@@ -300,6 +300,11 @@ ERR:
 	free(s);
 }
 
+static int ao_wav_get_latency(void)
+{
+	return 0;
+}
+
 static ao_info_t *ao_wav_get_driver_info(void)
 {
 	return &ao_wav_info;
@@ -328,5 +333,6 @@ ao_functions_t ao_wav =
         ao_wav_get_driver_info,
         ao_wav_open,
         ao_wav_play,
-        ao_wav_close
+        ao_wav_close,
+        ao_wav_get_latency
 };
Index: src/audio_out.c
===================================================================
RCS file: /usr/local/cvsroot/ao/src/audio_out.c,v
retrieving revision 1.9
diff -u -3 -p -r1.9 audio_out.c
--- src/audio_out.c	2000/12/17 20:28:35	1.9
+++ src/audio_out.c	2001/01/19 11:31:39
@@ -84,6 +84,8 @@ driver_tree_t *_get_plugin(char *plugin_
 		if (dlerror()) { free(dt->functions); free(dt); return NULL; }
 		dt->functions->close = dlsym(dt->handle, "plugin_close");
 		if (dlerror()) { free(dt->functions); free(dt); return NULL; }
+		dt->functions->get_latency = dlsym(dt->handle, "plugin_get_latency");
+		if (dlerror()) { free(dt->functions); free(dt); return NULL; }
 	} else {
 		return NULL;
 	}
@@ -318,3 +320,9 @@ int ao_is_big_endian(void) 
 	if (bytewise[0] == 0xba) return 1;
 	return 0;
 }
+
+int ao_get_latency(ao_device_t *device)
+{
+	return device->funcs->get_latency(device->state);
+}
+
Index: src/plugins/alsa/ao_alsa.c
===================================================================
RCS file: /usr/local/cvsroot/ao/src/plugins/alsa/ao_alsa.c,v
retrieving revision 1.2
diff -u -3 -p -r1.2 ao_alsa.c
--- src/plugins/alsa/ao_alsa.c	2000/10/30 04:38:48	1.2
+++ src/plugins/alsa/ao_alsa.c	2001/01/19 11:31:40
@@ -203,6 +203,14 @@ void plugin_play(ao_internal_t *state, v
 	}
 }
 
+int plugin_get_latency(ao_internal_t *state)
+{
+	ao_alsa_internal_t * s = (ao_alsa_internal_t *) state;
+	snd_pcm_channel_status_t status;
+	int err = snd_pcm_channel_status(s->pcm_handle, &status);
+	return (err < 0 ? 0 : status.count);
+}
+
 ao_info_t *plugin_get_driver_info(void)
 {
 	return &ao_alsa_info;
Index: src/plugins/arts/ao_arts.c
===================================================================
RCS file: /usr/local/cvsroot/ao/src/plugins/arts/ao_arts.c,v
retrieving revision 1.1
diff -u -3 -p -r1.1 ao_arts.c
--- src/plugins/arts/ao_arts.c	2000/12/30 05:03:25	1.1
+++ src/plugins/arts/ao_arts.c	2001/01/19 11:31:40
@@ -32,14 +32,17 @@
 typedef struct ao_arts_internal_s
 {
   arts_stream_t stream;
+	uint_32 bits;
+	uint_32 rate;
+	uint_32 channels;
 } ao_arts_internal_t;
 
 ao_info_t ao_arts_info =
 {
-	"aRts output",
-	"arts",
-	"Rik Hemsley (rikkus) <rik@kde.org>",
-	"Outputs to the aRts soundserver."
+  "aRts output",
+  "arts",
+  "Rik Hemsley (rikkus) <rik@kde.org>",
+  "Outputs to the aRts soundserver."
 };
 
   ao_internal_t *
@@ -74,7 +77,11 @@ plugin_open
 
   state->stream = arts_play_stream(rate, bits, channels, "ao stream");
 
-	return state;
+	state->bits = bits;
+	state->rate = rate;
+	state->channels = channels;
+
+  return state;
 }
 
   void
@@ -102,9 +109,18 @@ plugin_play
   }
 }
 
+  int
+plugin_get_latency(ao_internal_t * state)
+{
+  ao_arts_internal_t * s = (ao_arts_internal_t *)state;
+	int ms = arts_stream_get(s->stream, ARTS_P_TOTAL_LATENCY);
+	int sample_rate = (s->bits / 8) * s->rate * s->channels;
+	return (sample_rate * ms) / 1000;
+}
+
   ao_info_t *
 plugin_get_driver_info(void)
 {
-	return &ao_arts_info;
+  return &ao_arts_info;
 }
 
Index: src/plugins/esd/ao_esd.c
===================================================================
RCS file: /usr/local/cvsroot/ao/src/plugins/esd/ao_esd.c,v
retrieving revision 1.2
diff -u -3 -p -r1.2 ao_esd.c
--- src/plugins/esd/ao_esd.c	2000/10/30 04:38:48	1.2
+++ src/plugins/esd/ao_esd.c	2001/01/19 11:31:40
@@ -119,6 +119,11 @@ void plugin_play(ao_internal_t *state, v
 	write(((ao_esd_internal_t *) state)->sock, output_samples, num_bytes);
 }
 
+int plugin_get_latency(ao_internal_t *state)
+{
+	return (esd_get_latency((ao_esd_internal_t *)state->sock));
+}
+
 ao_info_t *plugin_get_driver_info(void)
 {
 	return &ao_esd_info;
Index: src/plugins/irix/ao_irix.c
===================================================================
RCS file: /usr/local/cvsroot/ao/src/plugins/irix/ao_irix.c,v
retrieving revision 1.2
diff -u -3 -p -r1.2 ao_irix.c
--- src/plugins/irix/ao_irix.c	2000/10/30 04:38:49	1.2
+++ src/plugins/irix/ao_irix.c	2001/01/19 11:31:41
@@ -163,6 +163,12 @@ void plugin_close(ao_internal_t *state)
 	free(state);
 }
 
+int plugin_get_latency(ao_internal_t *state)
+{
+	/* TODO */
+	return 0;
+}
+
 ao_info_t *plugin_get_driver_info(void)
 {
 	return &ao_irix_info;
Index: src/plugins/oss/ao_oss.c
===================================================================
RCS file: /usr/local/cvsroot/ao/src/plugins/oss/ao_oss.c,v
retrieving revision 1.2
diff -u -3 -p -r1.2 ao_oss.c
--- src/plugins/oss/ao_oss.c	2000/10/30 04:38:49	1.2
+++ src/plugins/oss/ao_oss.c	2001/01/19 11:31:41
@@ -129,7 +129,6 @@ ao_internal_t *plugin_open(uint_32 bits,
 	ioctl(state->fd,SNDCTL_DSP_SPEED, &tmp);
 	
 	return state;
-	
 
  ERR:
 	if(state != NULL)
@@ -158,6 +157,12 @@ void plugin_close(ao_internal_t *state)
 	free(s);
 }
 
+int plugin_get_latency(ao_internal_t *state)
+{
+	int odelay = 0;
+	ioctl(((ao_oss_internal_t *)state)->fd, SNDCTL_DSP_GETODELAY, &odelay);
+	return odelay;
+}
 
 ao_info_t *plugin_get_driver_info(void)
 {
Index: src/plugins/solaris/ao_solaris.c
===================================================================
RCS file: /usr/local/cvsroot/ao/src/plugins/solaris/ao_solaris.c,v
retrieving revision 1.2
diff -u -3 -p -r1.2 ao_solaris.c
--- src/plugins/solaris/ao_solaris.c	2000/10/30 04:38:49	1.2
+++ src/plugins/solaris/ao_solaris.c	2001/01/19 11:31:41
@@ -122,6 +122,12 @@ void plugin_close(ao_internal_t *state)
 	free(state);
 }
 
+int plugin_get_latency(ao_internal_t *state)
+{
+	/* TODO */
+	return 0;
+}
+
 const ao_info_t *plugin_get_driver_info(void)
 {
 	return &ao_solaris_info;

